
#ifndef KDEVPLATFORMLANGUAGE_EXPORT_H
#define KDEVPLATFORMLANGUAGE_EXPORT_H

#ifdef KDEVPLATFORMLANGUAGE_STATIC_DEFINE
#  define KDEVPLATFORMLANGUAGE_EXPORT
#  define KDEVPLATFORMLANGUAGE_NO_EXPORT
#else
#  ifndef KDEVPLATFORMLANGUAGE_EXPORT
#    ifdef KDevPlatformLanguage_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMLANGUAGE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMLANGUAGE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMLANGUAGE_NO_EXPORT
#    define KDEVPLATFORMLANGUAGE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMLANGUAGE_DEPRECATED
#  define KDEVPLATFORMLANGUAGE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMLANGUAGE_DEPRECATED_EXPORT
#  define KDEVPLATFORMLANGUAGE_DEPRECATED_EXPORT KDEVPLATFORMLANGUAGE_EXPORT KDEVPLATFORMLANGUAGE_DEPRECATED
#endif

#ifndef KDEVPLATFORMLANGUAGE_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMLANGUAGE_DEPRECATED_NO_EXPORT KDEVPLATFORMLANGUAGE_NO_EXPORT KDEVPLATFORMLANGUAGE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMLANGUAGE_NO_DEPRECATED
#    define KDEVPLATFORMLANGUAGE_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMLANGUAGE_EXPORT_H */
