/*
    SPDX-FileCopyrightText: 2008 Hamish Rodda <rodda@kde.org>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#ifndef KDEVPLATFORM_ALLTYPES_H
#define KDEVPLATFORM_ALLTYPES_H

#include "integraltype.h"
#include "functiontype.h"
#include "pointertype.h"
#include "referencetype.h"
#include "structuretype.h"
#include "delayedtype.h"
#include "arraytype.h"
#include "enumerationtype.h"
#include "enumeratortype.h"
#include "typealiastype.h"
#include "containertypes.h"

#endif
