
#ifndef KDEVPLATFORMOUTPUTVIEW_EXPORT_H
#define KDEVPLATFORMOUTPUTVIEW_EXPORT_H

#ifdef KDEVPLATFORMOUTPUTVIEW_STATIC_DEFINE
#  define KDEVPLATFORMOUTPUTVIEW_EXPORT
#  define KDEVPLATFORMOUTPUTVIEW_NO_EXPORT
#else
#  ifndef KDEVPLATFORMOUTPUTVIEW_EXPORT
#    ifdef KDevPlatformOutputView_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMOUTPUTVIEW_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMOUTPUTVIEW_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMOUTPUTVIEW_NO_EXPORT
#    define KDEVPLATFORMOUTPUTVIEW_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMOUTPUTVIEW_DEPRECATED
#  define KDEVPLATFORMOUTPUTVIEW_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMOUTPUTVIEW_DEPRECATED_EXPORT
#  define KDEVPLATFORMOUTPUTVIEW_DEPRECATED_EXPORT KDEVPLATFORMOUTPUTVIEW_EXPORT KDEVPLATFORMOUTPUTVIEW_DEPRECATED
#endif

#ifndef KDEVPLATFORMOUTPUTVIEW_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMOUTPUTVIEW_DEPRECATED_NO_EXPORT KDEVPLATFORMOUTPUTVIEW_NO_EXPORT KDEVPLATFORMOUTPUTVIEW_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMOUTPUTVIEW_NO_DEPRECATED
#    define KDEVPLATFORMOUTPUTVIEW_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMOUTPUTVIEW_EXPORT_H */
